package me.unurled.sacredrealms.sr.commands.admin;

import static me.unurled.sacredrealms.sr.utils.Component.comp;
import static me.unurled.sacredrealms.sr.utils.SRPlayerUtils.syncSRToPlayer;

import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;
import me.unurled.sacredrealms.sr.components.attributes.Attribute;
import me.unurled.sacredrealms.sr.components.player.PlayerManager;
import me.unurled.sacredrealms.sr.components.player.SRPlayer;
import me.unurled.sacredrealms.sr.gui.attributes.AttributesGUI;
import me.unurled.sacredrealms.sr.managers.Manager;
import org.bukkit.Bukkit;
import org.bukkit.command.Command;
import org.bukkit.command.CommandSender;
import org.bukkit.command.TabExecutor;
import org.bukkit.entity.Player;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import xyz.xenondevs.invui.window.Window;

public class AttributeCommand implements TabExecutor {

  public static final String FOR = " for ";
  public static final String REMOVE = "remove";

  @Nullable
  private static SRPlayer getSrPlayer(@NotNull CommandSender sender, Player target) {
    PlayerManager pm = Manager.getInstance(PlayerManager.class);
    if (pm == null) {
      sender.sendMessage(comp("<red>An error occurred."));
      return null;
    }
    SRPlayer player = pm.getPlayer(target.getUniqueId());
    if (player == null) {
      sender.sendMessage(comp("<red>An error occurred."));
      return null;
    }
    return player;
  }

  private static void remove(@NotNull CommandSender sender, Player target, Attribute attribute) {
    SRPlayer player = getSrPlayer(sender, target);
    if (player == null) return;
    player.removeAttribute(attribute);
    player.removeItemAttributes(attribute, target.getInventory().getItemInMainHand());
    syncSRToPlayer(player, target);
    sender.sendMessage("Removed " + attribute.getName() + FOR + target.getName());
    PlayerManager pm = Manager.getInstance(PlayerManager.class);
    pm.savePlayer(player);
  }

  private static void get(@NotNull CommandSender sender, Player target, Attribute attribute) {
    SRPlayer player = getSrPlayer(sender, target);
    if (player == null) return;
    sender.sendMessage(
        "Get "
            + attribute.getName()
            + FOR
            + target.getName()
            + ": "
            + player.getAttribute(attribute));
  }

  private static void set(
      @NotNull CommandSender sender,
      @NotNull String @NotNull [] args,
      Attribute attribute,
      Player target) {
    try {
      double value = Double.parseDouble(args[3]);
      if (value < attribute.getMinValue() || value > attribute.getMaxValue()) {
        sender.sendMessage(
            "Value out of bounds. Min: "
                + attribute.getMinValue()
                + ", Max: "
                + attribute.getMaxValue());
        return;
      }
      SRPlayer player = getSrPlayer(sender, target);
      if (player == null) return;
      player.setAttribute(attribute, value);
      sender.sendMessage("Set " + attribute.getName() + " to " + value + FOR + target.getName());
      player.setItemAttributes(attribute, target.getInventory().getItemInMainHand(), value);
      syncSRToPlayer(player, target);
      PlayerManager pm = Manager.getInstance(PlayerManager.class);
      pm.savePlayer(player);
    } catch (NumberFormatException e) {
      sender.sendMessage("Invalid value.");
    }
  }

  private static void itemInHand(@NotNull CommandSender sender, @NotNull String @NotNull [] args) {
    if (!(sender instanceof Player target)) {
      sender.sendMessage("You must be a player to use this command.");
      return;
    }
    Attribute attribute;
    try {
      attribute = Attribute.valueOf(args[2].toUpperCase());
    } catch (IllegalArgumentException e) {
      sender.sendMessage("Attribute not found.");
      return;
    }

    if (args[0].equalsIgnoreCase("set")) {
      try {
        Result result = getResult(sender, args, target, attribute);
        if (result == null) return;
        result.player().setHandItemAttribute(attribute, result.value());
        sender.sendMessage(
            "Set " + attribute.getName() + " to " + result.value() + FOR + target.getName());
        // update player
        result
            .player()
            .setItemAttributes(
                attribute, target.getInventory().getItemInMainHand(), result.value());
        syncSRToPlayer(result.player(), target);
      } catch (NumberFormatException e) {
        sender.sendMessage("Invalid value.");
      }
    } else if (args[0].equalsIgnoreCase("get")) {
      SRPlayer player = getSrPlayer(sender, target);
      if (player == null) return;
      sender.sendMessage(
          "Get "
              + attribute.getName()
              + FOR
              + target.getName()
              + ":"
              + " "
              + player.getHandItemAttribute(attribute));
    } else if (args[0].equalsIgnoreCase(REMOVE)) {
      SRPlayer player = getSrPlayer(sender, target);
      if (player == null) return;
      player.removeHandItemAttribute(attribute);
      player.removeItemAttributes(attribute, target.getInventory().getItemInMainHand());

      sender.sendMessage("Removed " + attribute.getName() + FOR + target.getName());
    } else {
      sender.sendMessage("Usage: /attribute <set|get|remove> <player> <attribute> [value]");
    }
  }

  private static void gui(@NotNull CommandSender sender, @NotNull String @NotNull [] args) {
    // open gui
    if (args.length != 2) {
      sender.sendMessage("Usage: /attribute gui <player>");
      return;
    }
    if (!(sender instanceof Player p)) {
      sender.sendMessage("You must be a player to use this command.");
      return;
    }
    Player target = Bukkit.getPlayer(args[1]);
    if (target == null) {
      sender.sendMessage("Player not found.");
      return;
    }
    Window window =
        Window.single()
            .setViewer(p)
            .setTitle("Attributes")
            .setGui(AttributesGUI.createGui(target))
            .build();
    window.open();
  }

  @Nullable
  private static Result getResult(
      @NotNull CommandSender sender,
      @NotNull String @NotNull [] args,
      Player target,
      Attribute attribute) {
    double value = Double.parseDouble(args[3]);
    if (value < attribute.getMinValue() || value > attribute.getMaxValue()) {
      sender.sendMessage(
          "Value out of bounds. Min: "
              + attribute.getMinValue()
              + ", Max: "
              + attribute.getMaxValue());
      return null;
    }
    SRPlayer player = getSrPlayer(sender, target);
    if (player == null) return null;
    return new Result(value, player);
  }

  /**
   * Executes the given command, returning its success. <br>
   * If false is returned, then the "usage" plugin.yml entry for this command (if defined) will be
   * sent to the player.
   *
   * @param sender Source of the command
   * @param command Command which was executed
   * @param label Alias of the command which was used
   * @param args Passed command arguments
   * @return true if a valid command, otherwise false
   */
  @Override
  public boolean onCommand(
      @NotNull CommandSender sender,
      @NotNull Command command,
      @NotNull String label,
      @NotNull String[] args) {
    // set, get, remove, gui
    if (!sender.hasPermission("sr.attributes")) {
      sender.sendMessage("Unknown command. Type \"/help\" for help.");
      return true;
    }
    if (args.length < 2) {
      sender.sendMessage(
          "Usage: /attribute <set|get|remove|gui> <player|item-in-hand> <attribute> [value]");
      return true;
    }
    if (args[0].equals("set") && args.length < 4) {
      sender.sendMessage("Usage: /attribute set <player|item-in-hand> <attribute> <value>");
      return true;
    }
    if (args[0].equals("gui")) {
      gui(sender, args);
      return true;
    }
    if (args[1].equals("item-in-hand")) {
      itemInHand(sender, args);
      return true;
    }
    Player target = Bukkit.getPlayer(args[1]);
    if (target == null) {
      sender.sendMessage("Player not found.");
      return true;
    }

    Attribute attribute;
    try {
      attribute = Attribute.valueOf(args[2].toUpperCase());
    } catch (IllegalArgumentException | ArrayIndexOutOfBoundsException e) {
      sender.sendMessage("Attribute not found.");
      return true;
    }

    if (args[0].equalsIgnoreCase("set")) {
      set(sender, args, attribute, target);
      return true;
    }
    if (args[0].equalsIgnoreCase("get")) {
      get(sender, target, attribute);
      return true;
    }
    if (args[0].equalsIgnoreCase(REMOVE)) {
      remove(sender, target, attribute);
      return true;
    }

    sender.sendMessage("Usage: /attribute <set|get|remove> <player> <attribute> [value]");
    return true;
  }

  /**
   * Requests a list of possible completions for a command argument.
   *
   * @param sender Source of the command. For players tab-completing a command inside of a command
   *     block, this will be the player, not the command block.
   * @param command Command which was executed
   * @param label Alias of the command which was used
   * @param args The arguments passed to the command, including final partial argument to be
   *     completed
   * @return A List of possible completions for the final argument, or null to default to the
   *     command executor
   */
  @Override
  public @Nullable List<String> onTabComplete(
      @NotNull CommandSender sender,
      @NotNull Command command,
      @NotNull String label,
      @NotNull String[] args) {
    if (!sender.hasPermission("sr.attributes")) {
      return null;
    }
    if (args.length == 1) {
      return List.of("set", "get", "gui", REMOVE);
    }
    if (args.length == 2) {
      List<String> players =
          Bukkit.getOnlinePlayers().stream().map(Player::getName).collect(Collectors.toList());
      players.add("item-in-hand");
      return players;
    }

    if (args.length == 3) {
      List<String> attributes =
          Arrays.stream(Attribute.values()).map(Attribute::getName).collect(Collectors.toList());
      attributes.removeIf(s -> !s.startsWith(args[2]));
      return attributes;
    }
    return null;
  }

  private record Result(double value, SRPlayer player) {}
}
