package me.unurled.sacredrealms.sr.components.entity;

import com.google.gson.annotations.Expose;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import me.unurled.sacredrealms.sr.components.attributes.Attribute;
import org.bukkit.entity.EntityType;
import org.bukkit.inventory.ItemStack;
import org.bukkit.potion.PotionEffect;

public class SREntityType {

  @Expose private final String ID;
  @Expose private String name;
  @Expose private Integer level = 1;
  @Expose private Long experience = 0L;
  @Expose private HashMap<Attribute, Double> attributes = new HashMap<>();
  @Expose private List<ItemStack> armor = new ArrayList<>();
  private HashMap<Attribute, HashMap<ItemStack, Double>> itemAttributes = new HashMap<>();
  @Expose private List<PotionEffect> potionEffects = new ArrayList<>();
  @Expose private HashMap<ItemStack, Double> lootTable = new HashMap<>();
  @Expose private EntityType type;
  @Expose private ItemStack item;
  @Expose private ItemStack handItem;
  @Expose private ItemStack secondHandItem;

  public SREntityType(String name, String ID) {
    this.name = name;
    this.ID = ID;
  }

  public String getName() {
    return name;
  }

  public void setName(String name) {
    this.name = name;
  }

  public String getID() {
    return ID;
  }

  public int getLevel() {
    return level;
  }

  public void setLevel(int level) {
    this.level = level;
  }

  public Long getExperience() {
    return experience;
  }

  public void setExperience(Long experience) {
    this.experience = experience;
  }

  public HashMap<Attribute, Double> getAttributes() {
    return attributes;
  }

  public void setAttributes(HashMap<Attribute, Double> attributes) {
    this.attributes = attributes;
  }

  public HashMap<Attribute, HashMap<ItemStack, Double>> getItemAttributes() {
    return itemAttributes;
  }

  public void setItemAttributes(HashMap<Attribute, HashMap<ItemStack, Double>> itemAttributes) {
    this.itemAttributes = itemAttributes;
  }

  public List<PotionEffect> getPotionEffects() {
    return potionEffects;
  }

  public void setPotionEffects(List<PotionEffect> potionEffects) {
    this.potionEffects = potionEffects;
  }

  public HashMap<ItemStack, Double> getLootTable() {
    return lootTable;
  }

  public void setLootTable(HashMap<ItemStack, Double> lootTable) {
    this.lootTable = lootTable;
  }

  public void setItemAttributes(Attribute attribute, ItemStack item, Double value) {
    if (itemAttributes.containsKey(attribute)) {
      itemAttributes.get(attribute).put(item, value);
    } else {
      HashMap<ItemStack, Double> map = new HashMap<>();
      map.put(item, value);
      itemAttributes.put(attribute, map);
    }
  }

  public void removeItemAttributes(Attribute attribute, ItemStack item) {
    if (itemAttributes.containsKey(attribute)) {
      itemAttributes.get(attribute).remove(item);
    }
  }

  public Double getItemAttributes(Attribute attribute, ItemStack item) {
    if (itemAttributes.containsKey(attribute)) {
      return itemAttributes.get(attribute).get(item);
    }
    return null;
  }

  public HashMap<ItemStack, Double> getItemAttributes(Attribute attribute) {
    return itemAttributes.getOrDefault(attribute, new HashMap<>());
  }

  public Double getTotalItemAttribute(Attribute attribute) {
    if (itemAttributes == null) itemAttributes = new HashMap<>();
    return itemAttributes.getOrDefault(attribute, new HashMap<>()).values().stream()
        .reduce(0.0, Double::sum);
  }

  public EntityType getType() {
    return type;
  }

  public void setType(EntityType type) {
    this.type = type;
  }

  public ItemStack getItem() {
    return item;
  }

  public void setItem(ItemStack item) {
    this.item = item;
  }

  public List<ItemStack> getArmor() {
    return armor;
  }

  public void setArmor(List<ItemStack> armor) {
    this.armor = armor;
  }

  public ItemStack getHandItem() {
    return handItem;
  }

  public void setHandItem(ItemStack handItem) {
    this.handItem = handItem;
  }

  public ItemStack getSecondHandItem() {
    return secondHandItem;
  }

  public void setSecondHandItem(ItemStack secondHandItem) {
    this.secondHandItem = secondHandItem;
  }
}
