package me.unurled.sacredrealms.sr.data;

import java.util.List;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

public interface DataHandler {

  /**
   * Get a value from the data source
   *
   * @param key The key to get the value from
   * @return The value of the key
   */
  String get(@NotNull String key);

  /**
   * Set a value in the data source
   *
   * @param key The key to set the value to
   * @param value The value to set
   */
  void set(@NotNull String key, @Nullable String value);

  /**
   * Remove a value from the data source
   *
   * @param key The key to remove the value from
   */
  void remove(@NotNull String key);

  /**
   * Check if a key exists in the data source
   *
   * @param key The key to check
   * @return If the key exists
   */
  boolean exists(@NotNull String key);

  /**
   * Get all keys from the data source
   *
   * @return All keys
   */
  List<String> getKeysAll(@NotNull String key);
}
