package me.unurled.sacredrealms.sr.gui.entitytype;

import java.util.List;
import java.util.stream.Collectors;
import me.unurled.sacredrealms.sr.components.entity.EntityManager;
import me.unurled.sacredrealms.sr.components.entity.SREntityType;
import me.unurled.sacredrealms.sr.gui.BackItem;
import me.unurled.sacredrealms.sr.gui.ForwardItem;
import org.bukkit.Material;
import org.jetbrains.annotations.NotNull;
import xyz.xenondevs.invui.gui.Gui;
import xyz.xenondevs.invui.gui.PagedGui;
import xyz.xenondevs.invui.gui.structure.Markers;
import xyz.xenondevs.invui.item.Item;
import xyz.xenondevs.invui.item.builder.ItemBuilder;
import xyz.xenondevs.invui.item.impl.SimpleItem;

public class EntityTypeGUI {
  public static @NotNull Gui listGui() {
    EntityManager em = EntityManager.getInstance(EntityManager.class);
    List<Item> items =
        em.getTypes().stream()
            .map(type -> new SimpleItem(new ItemBuilder(type.getItem())))
            .collect(Collectors.toList());

    Gui gui =
        PagedGui.items()
            .setStructure(
                "# # # # # # # # #",
                "# X X X X X X X #",
                "# X X X X X X X #",
                "# X X X X X X X #",
                "# # # < # > # # #")
            .addIngredient('X', Markers.CONTENT_LIST_SLOT_HORIZONTAL)
            .addIngredient('<', new BackItem())
            .addIngredient('>', new ForwardItem())
            .setContent(items)
            .build();
    return gui;
  }

  public static @NotNull Gui createGui(SREntityType type) {

    Item border =
        new SimpleItem(new ItemBuilder(Material.BLACK_STAINED_GLASS_PANE).setDisplayName("§r"));

    Gui gui =
        Gui.normal()
            .setStructure(
                "# # # # # # # # #",
                "# T B O I S N L #", // Type, Behavior, lOot, Item display, Stats, Name, Level
                "# X X X P X X X #", // exPerience
                "# H C E B X M A #", // Helmet, Chestplate, lEgging, Boots, Main hand, second hAnd
                "# # # # # # # # #")
            .addIngredient('#', border)
            .addIngredient('T', new EntityTypeItem(type))
            .addIngredient('B', new EntityBehaviorItem(type))
            .addIngredient('O', new EntityLootItem())
            .addIngredient('I', new EntityItemDisplay(type))
            .addIngredient('S', new EntityStatsItem(type))
            .addIngredient('N', new EntityNameItem(type))
            .addIngredient('L', new EntityLevelItem(type))
            .addIngredient('P', new EntityExpItem(type))
            .addIngredient('H', new EntityArmorItem(type, 0))
            .addIngredient('C', new EntityArmorItem(type, 1))
            .addIngredient('E', new EntityArmorItem(type, 2))
            .addIngredient('B', new EntityArmorItem(type, 3))
            .addIngredient('M', new EntityArmorItem(type, 4))
            .addIngredient('A', new EntityArmorItem(type, 5))
            .build();
    return gui;
  }
}
